html, body {
    height: 100%;
    width: 100%;
}

.navbar-inverse {
  background-color: rgba(0,0,0,.7);
  color: #cecece;
}

#map {
    height: 100%;
    width: 100%;
    left: 0;
    position: relative;
    top: 0;
}

.tooltip {
    position: relative;
    background: rgba(0, 0, 0, 0.5);
    border-radius: 4px;
    color: white;
    padding: 4px 8px;
    opacity: 0.7;
    white-space: nowrap;
}
.tooltip-measure {
    opacity: 1;
    font-weight: bold;
}
.tooltip-static {
    background-color: #ffcc33;
    color: black;
    border: 1px solid white;
}
.tooltip-measure:before,
.tooltip-static:before {
    border-top: 6px solid rgba(0, 0, 0, 0.5);
    border-right: 6px solid transparent;
    border-left: 6px solid transparent;
    content: "";
    position: absolute;
    bottom: -6px;
    margin-left: -7px;
    left: 50%;
}
.tooltip-static:before {
    border-top-color: #ffcc33;
}

.ol-zoom.ol-unselectable.ol-control {
    bottom: 7px;
    top: auto;
}

.childMenu .btn-toolbar {
    padding: 5px 0;
}

.btn-active {
    background-color: #337ab7;
}

@media (max-width: 767px) {
    .childMenu {
        margin: 0 -15px;
        border-top: 1px solid #101010;
        padding: 0 15px;
    }

    .childMenu .navbar-form {
        border: none;
        margin: 0;
        padding: 15px;
        clear: both;
    }

    .childMenu li {
        clear: both;
    }

    .navbar-fixed-top {
        overflow-y: auto;
        max-height: 100%;
    }

}

